
#ifndef _FUNCPTR_HPP_
#define _FUNCPTR_HPP_

namespace ps
{
	/**
	 * \brief Class encapsulating a function pointer
	 *
	 * The class automatically prevents calling of null pointers.
	 */
	class FuncPtr
	{
		public:
			FuncPtr(void *addr = 0)
			{
				this->addr = addr;
			}
			
			void set(void *addr)
			{
				this->addr = addr;
			}
			void *get(void)
			{
				return addr;
			}
			
			void call(void)
			{
				if (!addr) return;
				typedef void (*FUNCPTR)(void);
				void (*funcptr)(void);
				funcptr = (FUNCPTR)addr;
				funcptr();
			}
			template <typename A> void call(A arg1)
			{
				if (!addr) return;
				typedef void (*FUNCPTR)(A);
				void (*funcptr)(A);
				funcptr = (FUNCPTR)addr;
				funcptr(arg1);
			}
			template <typename A, typename B> void call(A arg1, B arg2)
			{
				if (!addr) return;
				typedef void (*FUNCPTR)(A, B);
				void (*funcptr)(A, B);
				funcptr = (FUNCPTR)addr;
				funcptr(arg1, arg2);
			}
			template <typename A, typename B, typename C> void call(A arg1, B arg2, C arg3)
			{
				if (!addr) return 0;
				typedef void (*FUNCPTR)(A, B, C);
				void (*funcptr)(A, B, C);
				funcptr = (FUNCPTR)addr;
				funcptr(arg1, arg2, arg3);
			}
			template <typename R> R call(void)
			{
				if (!addr) return 0;
				typedef R (*FUNCPTR)(void);
				R (*funcptr)(void);
				funcptr = (FUNCPTR)addr;
				return funcptr();
			}
			template <typename R, typename A> R call(A arg1)
			{
				if (!addr) return 0;
				typedef R (*FUNCPTR)(A);
				R (*funcptr)(A);
				funcptr = (FUNCPTR)addr;
				return funcptr(arg1);
			}
			template <typename R, typename A, typename B> R call(A arg1, B arg2)
			{
				if (!addr) return 0;
				typedef R (*FUNCPTR)(A, B);
				R (*funcptr)(A, B);
				funcptr = (FUNCPTR)addr;
				return funcptr(arg1, arg2);
			}
			template <typename R, typename A, typename B, typename C> R call(A arg1, B arg2, C arg3)
			{
				if (!addr) return 0;
				typedef R (*FUNCPTR)(A, B, C);
				R (*funcptr)(A, B, C);
				funcptr = (FUNCPTR)addr;
				return funcptr(arg1, arg2, arg3);
			}
		private:
			void *addr;
	};
}

#endif

