
#ifndef _MODULE_HPP_
#define _MODULE_HPP_

struct module_t;

namespace ps
{
	class VM;
	
	/**
	 * \brief Class which holds a single module.
	 *
	 * A module consists of one script file, multiple modules are connected
	 * wihtin the VM, functions can be called across different modules.
	 */
	class Module
	{
		public:
			Module(VM *vm);
			~Module();
			
			/**
			 * \brief Loads the module from a script file.
			 *
			 * Do not use this function directly, use VM::loadModule() instead.
			 */
			bool loadSource(const char *filename);
			/**
			 * \brief Loads the module from a file holding the parsed bytecode.
			 *
			 * Do not use this function directly, use VM::loadBytecode() instead.
			 */
			bool loadBytecode(const char *filename);
			
			/**
			 * \brief Saves the parsed bytecode to a file.
			 *
			 * Bytecode can be loaded much faster than source files which need
			 * to be parsed first, however, there can be some caveats when the
			 * environment while parsing the source differs from the environment
			 * used to compile the bytecode.
			 */
			bool saveBytecode(const char *filename);
			
			/**
			 * \brief Sends a listing of the bytecode to stdout.
			 */
			void displayBytecode(void);
		private:
			VM *vm;
			module_t *mod;
	};
}

#endif

