
#ifndef _PLUSSCRIPT_HPP_
#define _PLUSSCRIPT_HPP_

#include "vm.hpp"

/**
 * \mainpage PlusScript
 * \section intro_sec Introduction
 * PlusScript is a simple scripting language. It bases on a just in time
 * compiler which automatically compiles a function to native machine code when
 * it is called the first time. Afterwards, the function can be executed without
 * much overhead, therefore the language is much faster than other "normal"
 * scripting languages which are based on an interpreter.
 * \section example_sec Very simple example
 * Script file:
 * \code
 * function hello()
 *     print("Hello World!");
 * end
 * \endcode
 * Program to call the file:
 * \code
 * #include "plusscript.hpp"
 * using namespace ps;
 * 
 * // Output function
 * void print(char *msg)
 * {
 *     printf("%s\n", msg);
 * }
 *
 * int main(int argc, char **argv)
 * {
 *     VM::setLoggingOutput("plusscript.log");
 *     // Create VM
 *     VM vm;
 *     vm.registerBaseFunctions();
 *     // Register external function
 *     std::vector<TypeInfo> args;
 *     args.push_back(TypeInfo(ETC_UserPtr));
 *     vm.registerFunction("print", (void*)print, TypeInfo(ETC_None), args);
 *     // Load script file
 *     Module *mod = vm.loadModule("helloworld.ps");
 *     // Call script function
 *     vm.getFunction("hello").call();
 *     return 1;
 * }
 * \endcode
 */

/**
 * \brief Namespace which holds all the classes of the scripting engine
 */
namespace ps
{
}

#endif

