
#ifndef _TYPEINFO_HPP_
#define _TYPEINFO_HPP_

#include <string>

namespace ps
{
	enum TypeClass
	{
		ETC_Integer,
		ETC_Float,
		ETC_UserPtr,
		ETC_Struct,
		ETC_None
	};
	/**
	 * \brief Class holding a variable type.
	 *
	 * It is made up from two parts: There is the general variable class
	 * (integer, float, structure, pointer) and a type name which identifies
	 * the struct.
	 */
	class TypeInfo
	{
		public:
			TypeInfo()
			{
				type = ETC_None;
			}
			TypeInfo(TypeClass type, std::string name = "")
			{
				this->type = type;
				this->name = name;
			}
			
			TypeClass type;
			std::string name;
	};
	
	/**
	 * \brief Converts a type class to a variable type used internally by the
	 * engine.
	 */
	int getVariableType(TypeClass typeclass);
	/**
	 * \brief Converts a variable type to a type class.
	 */
	TypeClass getTypeClass(int vartype);
}

#endif

