
#include "vm.hpp"

#include "plusscript.h"

#include <stdio.h>

typedef void (*CONSTRUCTOR)(void*);
typedef void (*OPEQUAL)(void*, int, char*, void*);

namespace ps
{
	VM::VM()
	{
		vm = create_vm();
	}
	VM::~VM()
	{
		// Delete modules
		for (unsigned int i = 0; i < modules.size(); i++)
		{
			delete modules[i];
		}
		// Delete struct info
		for (unsigned int i = 0; i < structinfo.size(); i++)
		{
			if (structinfo[i]->membercount > 0)
			{
				for (int j = 0; j < structinfo[i]->membercount; j++)
				{
					free(structinfo[i]->members[j].name);
					free(structinfo[i]->members[j].type_name);
				}
				delete[] structinfo[i]->members;
			}
			if (structinfo[i]->funccount > 0)
			{
				for (int j = 0; j < structinfo[i]->funccount; j++)
				{
					free(structinfo[i]->functions[j].name);
					if (structinfo[i]->functions[j].argcount > 0)
					{
						delete[] structinfo[i]->functions[j].argtypes;
					}
				}
				delete[] structinfo[i]->functions;
			}
			free(structinfo[i]->name);
			delete structinfo[i];
		}
		// Delete the vm itself
		delete_vm(vm);
	}
	
	Module *VM::loadModule(const char *filename)
	{
		Module *newmod = new Module(this);
		if (!newmod->loadSource(filename))
		{
			delete newmod;
			return 0;
		}
		modules.push_back(newmod);
		return newmod;
	}
	
	Module *VM::loadBytecode(const char *filename)
	{
		return 0;
	}
	
	FuncPtr VM::getFunction(const char *name)
	{
		return FuncPtr(vm_get_function_address(vm, name));
	}
	
	void VM::registerFunction(std::string name, void *addr,
		TypeInfo rettype,
		std::vector<TypeInfo> arguments)
	{
		if (arguments.size() > 0)
		{
			int *argtypes = new int[arguments.size()];
			for (unsigned int i = 0; i < arguments.size(); i++)
			{
				argtypes[i] = getVariableType(arguments[i].type);
			}
			vm_register_function(vm, addr, name.c_str(), arguments.size(), argtypes, getVariableType(rettype.type));
			delete[] argtypes;
		}
		else
		{
			vm_register_function(vm, addr, name.c_str(), 0, 0, getVariableType(rettype.type));
		}
	}
	
	void VM::registerStruct(std::string name, StructInfo info)
	{
		struct_info_t *structinfo = new struct_info_t;
		structinfo->name = strdup(name.c_str());
		structinfo->size = info.size;
		structinfo->constructor = (CONSTRUCTOR)info.constructor;
		structinfo->destructor = (CONSTRUCTOR)info.destructor;
		structinfo->op_equal = (OPEQUAL)info.opequal;
		// Add members
		structinfo->membercount = info.members.size();
		structinfo->members = new member_var_t[info.members.size()];
		for (unsigned int i = 0; i < info.members.size(); i++)
		{
			structinfo->members[i].name = strdup(info.members[i].name.c_str());
			structinfo->members[i].type = getVariableType(info.members[i].type.type);
			structinfo->members[i].type_name = 0;
			if (info.members[i].type.name != "")
			{
				structinfo->members[i].type_name = strdup(info.members[i].type.name.c_str());
			}
			structinfo->members[i].offset = info.members[i].offset;
			structinfo->members[i].size = info.members[i].size;
		}
		// Add functions
		structinfo->funccount = info.functions.size();
		structinfo->functions = new ext_func_t[info.functions.size()];
		for (unsigned int i = 0; i < info.functions.size(); i++)
		{
			structinfo->functions[i].name = strdup(info.functions[i].name.c_str());
			structinfo->functions[i].function = info.functions[i].function;
			structinfo->functions[i].argcount = info.functions[i].arguments.size();
			structinfo->functions[i].argtypes = new int[info.functions[i].arguments.size()];
			if (info.functions[i].arguments.size() > 0)
			{
				for (unsigned int j = 0; j < info.functions[i].arguments.size(); j++)
				{
					structinfo->functions[i].argtypes[j] = getVariableType(info.functions[i].arguments[j].type);
				}
			}
			structinfo->functions[i].returntype = getVariableType(info.functions[i].rettype.type);
		}
		// Register struct
		vm_register_struct(vm, structinfo);
		this->structinfo.push_back(structinfo);
	}
	
	void VM::registerBaseFunctions(void)
	{
		vm_register_base(vm);
	}
	
	void VM::setLoggingOutput(const char *filename)
	{
		if (logfile) fclose((FILE*)logfile);
		if (filename)
		{
			logfile = fopen(filename, "wt");
		}
		else
		{
			logfile = 0;
		}
		set_logging_output((FILE*)logfile);
	}
	
	ps_vm_t *VM::getVM(void)
	{
		return vm;
	}
	
	void *VM::logfile = 0;
}

