
/*
Copyright (c) 2008 Mathias Gottschlag

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
*/

#ifndef _AMD64_H_
#define _AMD64_H_

#ifdef __cplusplus
extern "C" {
#endif

#ifdef __amd64__

#include "module.h"

#define REG_RAX 1
#define REG_RCX 2
#define REG_RDX 3
#define REG_RSI 4
#define REG_RDI 5
#define REG_XMM0 10
#define REG_XMM1 11
#define REG_XMM2 12
#define REG_XMM3 13

void amd64_set_local_var(module_t *module, int var, int reg);
void amd64_get_local_var(module_t *module, int var, int reg);
void amd64_get_offset(module_t *module, int addrreg, int offset, int destreg, int size);
void amd64_set_offset(module_t *module, int addrreg, int offset, int valuereg, int size);
void amd64_call_function(module_t *module, void *function);
void amd64_add(module_t *module, int reg1, int reg2, int type);
void amd64_sub(module_t *module, int reg1, int reg2, int type);
void amd64_mul(module_t *module, int reg1, int reg2, int type);
void amd64_div(module_t *module, int reg1, int reg2, int type);
void amd64_and(module_t *module, int reg1, int reg2);
void amd64_or(module_t *module, int reg1, int reg2);
void amd64_xor(module_t *module, int reg1, int reg2);
void amd64_compare(module_t *module, int reg1, int reg2, int type, int vartype);

void amd64_convert(module_t *module, int srctype, int desttype);

void amd64_set_int_argument(module_t *module, int idx);
void amd64_set_float_argument(module_t *module, int idx);
void amd64_get_int_argument(module_t *module, int var, int idx);
void amd64_get_float_argument(module_t *module, int var, int idx);

void amd64_call_operator(module_t *module, int varidx, void *function, int type, char *name, int local);

#endif

#ifdef __cplusplus
}
#endif

#endif

