
/*
Copyright (c) 2008 Mathias Gottschlag

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
*/

#ifndef _BYTECODE_H_
#define _BYTECODE_H_

#ifdef __cplusplus
extern "C" {
#endif

#include "module.h"

#define CMD_IF 1
#define CMD_LABEL 2
#define CMD_JMP 3
#define CMD_SET 4
#define CMD_SETMEMBER 17
#define CMD_GET 5
#define CMD_GETMEMBER 18
#define CMD_GETC 10
#define CMD_GETF 20
#define CMD_GETS 13
#define CMD_ADD 6
#define CMD_SUB 7
#define CMD_MUL 11
#define CMD_DIV 12
#define CMD_CONCAT 14
#define CMD_AND 21
#define CMD_OR 22
#define CMD_XOR 23
#define CMD_CMP 15
#define CMD_CALL 8
#define CMD_CALLMEMBER 19
#define CMD_END 9
#define CMD_RETURN 16

// IF:
#define CMP_EQUAL 0x1
#define CMP_GREATER 0x2
#define CMP_SMALLER 0x4

static inline void bytecode_add_label(module_t *module, int id)
{
	module->bytecode[module->codelen++] = CMD_LABEL;
	*((int*)(module->bytecode + module->codelen)) = id;
	module->codelen += 4;
}
static inline void bytecode_add_jmp(module_t *module, int id)
{
	module->bytecode[module->codelen++] = CMD_JMP;
	*((int*)(module->bytecode + module->codelen)) = id;
	module->codelen += 4;
}
static inline void bytecode_add_get(module_t *module, int id)
{
	module->bytecode[module->codelen++] = CMD_GET;
	*((int*)(module->bytecode + module->codelen)) = id;
	module->codelen += 4;
}
static inline void bytecode_add_getmember(module_t *module, int id)
{
	module->bytecode[module->codelen++] = CMD_GETMEMBER;
	*((int*)(module->bytecode + module->codelen)) = id;
	module->codelen += 4;
}
static inline void bytecode_add_getc(module_t *module, int value)
{
	module->bytecode[module->codelen++] = CMD_GETC;
	*((int*)(module->bytecode + module->codelen)) = value;
	module->codelen += 4;
}
static inline void bytecode_add_getf(module_t *module, float value)
{
	module->bytecode[module->codelen++] = CMD_GETF;
	*((float*)(module->bytecode + module->codelen)) = value;
	module->codelen += 4;
}
static inline void bytecode_add_gets(module_t *module, int id)
{
	module->bytecode[module->codelen++] = CMD_GETS;
	*((int*)(module->bytecode + module->codelen)) = id;
	module->codelen += 4;
}
static inline void bytecode_add_set(module_t *module, int id)
{
	module->bytecode[module->codelen++] = CMD_SET;
	*((int*)(module->bytecode + module->codelen)) = id;
	module->codelen += 4;
}
static inline void bytecode_add_setmember(module_t *module, int id)
{
	module->bytecode[module->codelen++] = CMD_SETMEMBER;
	*((int*)(module->bytecode + module->codelen)) = id;
	module->codelen += 4;
}
static inline void bytecode_add_if(module_t *module, int id)
{
	module->bytecode[module->codelen++] = CMD_IF;
	*((int*)(module->bytecode + module->codelen)) = id;
	module->codelen += 4;
}
static inline void bytecode_add_cmp(module_t *module, char type)
{
	module->bytecode[module->codelen++] = CMD_CMP;
	module->bytecode[module->codelen++] = type;
}
static inline void bytecode_add_callmember(module_t *module, int id, char argcount, char retval)
{
	module->bytecode[module->codelen++] = CMD_CALLMEMBER;
	*((int*)(module->bytecode + module->codelen)) = id;
	module->codelen += 4;
	module->bytecode[module->codelen++] = argcount;
	module->bytecode[module->codelen++] = retval;
}
static inline void bytecode_add_call(module_t *module, int id, char argcount, char retval)
{
	module->bytecode[module->codelen++] = CMD_CALL;
	*((int*)(module->bytecode + module->codelen)) = id;
	module->codelen += 4;
	module->bytecode[module->codelen++] = argcount;
	module->bytecode[module->codelen++] = retval;
}
static inline void bytecode_add_return(module_t *module, char retval)
{
	module->bytecode[module->codelen++] = CMD_RETURN;
	module->bytecode[module->codelen++] = retval;
}

#ifdef __cplusplus
}
#endif

#endif

