
/*
Copyright (c) 2008 Mathias Gottschlag

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
*/

#ifndef _MODULE_H_
#define _MODULE_H_

#ifdef __cplusplus
extern "C" {
#endif

struct ps_vm_t;

#define VAR_TYPE_UNKNOWN 0
#define VAR_TYPE_INT 1
#define VAR_TYPE_FLOAT 2
#define VAR_TYPE_STRUCT 3
#define VAR_TYPE_USERPTR 4

typedef struct mod_var_t
{
	int type;
	int type_name;
	int name;
	int value;
	void *addr;
} mod_var_t;

typedef struct mod_func_t
{
	int name;
	int status;
	int bcoffset;
	int progoffset;
	
	int *argnames;
	int argcount;
	
	mod_var_t *vars;
	int varcount;
	
	char *call_stub;
	
	int rettype;
	int rettypename;
} mod_func_t;

typedef struct mod_type_t
{
	int name;
	mod_var_t *fields;
	int field_count;
} mod_type_t;

typedef struct module_t
{
	char bytecode[4096];
	int codelen;
	
	char *program;
	int proglen;
	
	char data[1024];
	
	mod_func_t *functions;
	int funccount;
	
	mod_var_t *vars;
	int varcount;
	
	mod_type_t *usertypes;
	int typecount;
	
	char **strings;
	int stringcount;
} module_t;

#define CTRL_FRAME_IF 1
#define CTRL_FRAME_ELSE 2
#define CTRL_FRAME_ELSEIF 3
#define CTRL_FRAME_FOR 4
#define CTRL_FRAME_WHILE 5

typedef struct ctrl_frame_t
{
	int type;
	int id;
	int value1;
	int value2;
} ctrl_frame_t;

int get_string(module_t *module, const char *string);
int get_variable_index(mod_func_t *function, int name);
int var_is_valid(module_t *module, mod_func_t *function, int name);

module_t *parse_source(struct ps_vm_t *vm, const char *srcfile);
int save_module(module_t *module, const char *file);
void delete_module(struct ps_vm_t *vm, module_t *module);
void display_bytecode(module_t *module);

#ifdef __cplusplus
}
#endif

#endif

