
/*
Copyright (c) 2008 Mathias Gottschlag

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
*/

#ifndef _VM_H_
#define _VM_H_

#include "module.h"

#ifdef __cplusplus
extern "C" {
#endif

typedef struct ext_func_t
{
	char *name;
	void *function;
	int argcount;
	int *argtypes;
	int returntype;
} ext_func_t;

typedef struct member_var_t
{
	char *name;
	int type;
	char *type_name;
	int offset;
	int size;
} member_var_t;

typedef void (*CONSTRUCTOR_PTR)(void*);
typedef void (*DESTRUCTOR_PTR)(void*);
typedef void (*OPERATOR_EQ_PTR)(void*, int, char*, void*);
typedef struct struct_info_t
{
	char *name;
	int size;
	
	void (*constructor)(void*);
	void (*destructor)(void*);
	void (*op_equal)(void*, int, char*, void*);
	
	int membercount;
	member_var_t *members;
	
	int funccount;
	ext_func_t *functions;
} struct_info_t;

typedef struct ps_vm_t
{
	module_t **modules;
	int modcount;
	
	ext_func_t *functions;
	int funccount;
	
	struct_info_t **structs;
	int structcount;
} ps_vm_t;

ps_vm_t *create_vm(void);
void delete_vm(ps_vm_t *vm);

void vm_register_function(ps_vm_t *vm, void *function, const char *name, int argcount, int *argtypes, int rettype);
void vm_register_struct(ps_vm_t *vm, struct_info_t *s);

void vm_add_module(ps_vm_t *vm, module_t *module);
void vm_call_function(ps_vm_t *vm, const char *name);
void *vm_get_function_address(ps_vm_t *vm, const char *name);
struct_info_t *vm_get_struct_info(ps_vm_t *vm, const char *name);

void vm_register_base(ps_vm_t *vm);

#ifdef __cplusplus
}
#endif

#endif

