
/*
Copyright (c) 2008 Mathias Gottschlag

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
*/

#ifndef _X86_H_
#define _X86_H_

#ifdef __cplusplus
extern "C" {
#endif

#ifdef __i386__

#include "module.h"

#define REG_EAX 1
#define REG_ECX 2
#define REG_EDX 3
#define REG_ESI 4
#define REG_EDI 5

void x86_set_local_var(module_t *module, int var, int reg);
void x86_get_local_var(module_t *module, int var, int reg);
void x86_get_offset(module_t *module, int addrreg, int offset, int destreg, int size);
void x86_set_offset(module_t *module, int addrreg, int offset, int valuereg, int size);
void x86_call_function(module_t *module, void *function);
void x86_add(module_t *module, int reg1, int reg2, int type);
void x86_sub(module_t *module, int reg1, int reg2, int type);
void x86_mul(module_t *module, int reg1, int reg2, int type);
void x86_div(module_t *module, int reg1, int reg2, int type);
void x86_and(module_t *module, int reg1, int reg2);
void x86_or(module_t *module, int reg1, int reg2);
void x86_xor(module_t *module, int reg1, int reg2);
void x86_compare(module_t *module, int reg1, int reg2, int type, int vartype);

void x86_adjust_esp(module_t *module, int size);

void x86_convert(module_t *module, int srctype, int desttype);

void x86_flip_args(module_t *module, int count, int skip, int convert, int *types, int *expected);

void x86_call_operator(module_t *module, int varidx, void *function, int type, char *name, int local);

#endif

#ifdef __cplusplus
}
#endif

#endif

