
/*
Copyright (c) 2008 Mathias Gottschlag

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
*/

#include "vm.h"

#include <string.h>
#include <stdlib.h>

// String struct
typedef struct ps_string_t
{
	char *strdata;
} ps_string_t;

void stringconstructor(ps_string_t *s)
{
	s->strdata = strdup("");
}
void stringdestructor(ps_string_t *s)
{
	free(s->strdata);
}
void stringequal(ps_string_t *s, int type, char *type_name, void *data)
{
	if (type == VAR_TYPE_USERPTR)
	{
		free(s->strdata);
		s->strdata = strdup((char*)data);
	}
	else if ((type == VAR_TYPE_STRUCT) && !strcmp(type_name, "string"))
	{
		free(s->strdata);
		s->strdata = strdup(((ps_string_t*)data)->strdata);
	}
}

char *string_c_str(ps_string_t *s)
{
	return s->strdata;
}

int string_cmp(ps_string_t *s, char *s2)
{
	return !strcmp(s->strdata, s2);
}
int string_find(ps_string_t *s, char *s2)
{
	char *found = strstr(s->strdata, s2);
	if (found)
	{
		return found - s->strdata;
	}
	else
	{
		return -1;
	}
}
int string_findc(ps_string_t *s, char c)
{
	char *found = strchr(s->strdata, c);
	if (found)
	{
		return found - s->strdata;
	}
	else
	{
		return -1;
	}
}
char *string_cut(ps_string_t *s, int begin, int end)
{
	char *newstr = (char*)malloc(end - begin + 1);
	memcpy(newstr, s->strdata + begin, end - begin);
	newstr[end - begin] = 0;
	free(s->strdata);
	s->strdata = newstr;
	return s->strdata;
}
int cmpargtypes[1] = {VAR_TYPE_USERPTR};
int findargtypes[1] = {VAR_TYPE_USERPTR};
int findcargtypes[1] = {VAR_TYPE_INT};
int cutargtypes[2] = {VAR_TYPE_INT, VAR_TYPE_INT};

ext_func_t stringfunctions[5] =
{
	{
		"c_str",
		(void*)string_c_str,
		0,
		0,
		VAR_TYPE_USERPTR
	},
	{
		"find",
		(void*)string_find,
		1,
		findargtypes,
		VAR_TYPE_INT
	},
	{
		"findc",
		(void*)string_findc,
		1,
		findcargtypes,
		VAR_TYPE_INT
	},
	{
		"cut",
		(void*)string_cut,
		2,
		cutargtypes,
		VAR_TYPE_USERPTR
	},
	{
		"cmp",
		(void*)string_cmp,
		1,
		cmpargtypes,
		VAR_TYPE_INT
	}
};

struct_info_t stringinfo =
{
	"string",
	sizeof(ps_string_t),
	(CONSTRUCTOR_PTR)stringconstructor,
	(DESTRUCTOR_PTR)stringdestructor,
	(OPERATOR_EQ_PTR)stringequal,
	0,
	0,
	5,
	stringfunctions
};

void vm_register_base(ps_vm_t *vm)
{
	vm_register_struct(vm, &stringinfo);
}

