
/*
Copyright (c) 2008 Mathias Gottschlag

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
*/

#include "bytecode.h"
#include "module.h"

#include <stdio.h>

void display_bytecode(module_t *module)
{
	printf("\nBytecode:\n");
	int pos = 0;
	while (pos < module->codelen)
	{
		switch (module->bytecode[pos])
		{
			case CMD_IF:
			{
				int label = *((int*)(&module->bytecode[pos + 1]));
				printf("%04d: IF (%d)\n", pos, label);
				pos += 5;
				break;
			}
			case CMD_CMP:
			{
				printf("%04d: CMP (", pos);
				char type = module->bytecode[pos + 1];
				if (type & CMP_SMALLER) printf("<");
				if (type & CMP_GREATER) printf(">");
				if (type & CMP_EQUAL) printf("=");
				if (type == CMP_EQUAL) printf("=");
				printf(")\n");
				
				pos += 2;
				break;
			}
			case CMD_LABEL:
			{
				int label = *((int*)(&module->bytecode[pos + 1]));
				printf("%04d: LABEL (%d)\n", pos, label);
				pos += 5;
				break;
			}
			case CMD_JMP:
			{
				int label = *((int*)(&module->bytecode[pos + 1]));
				printf("%04d: JMP (label: %d)\n", pos, label);
				pos += 5;
				break;
			}
			case CMD_SET:
			{
				int var = *((int*)(&module->bytecode[pos + 1]));
				printf("%04d: SET (%s)\n", pos, module->strings[var]);
				pos += 5;
				break;
			}
			case CMD_SETMEMBER:
			{
				int var = *((int*)(&module->bytecode[pos + 1]));
				printf("%04d: SETMEMBER (%s)\n", pos, module->strings[var]);
				pos += 5;
				break;
			}
			case CMD_GET:
			{
				int var = *((int*)(&module->bytecode[pos + 1]));
				printf("%04d: GET (%s)\n", pos, module->strings[var]);
				pos += 5;
				break;
			}
			case CMD_GETMEMBER:
			{
				int var = *((int*)(&module->bytecode[pos + 1]));
				printf("%04d: GETMEMBER (%s)\n", pos, module->strings[var]);
				pos += 5;
				break;
			}
			case CMD_GETC:
			{
				int var = *((int*)(&module->bytecode[pos + 1]));
				printf("%04d: GETC (%d)\n", pos, var);
				pos += 5;
				break;
			}
			case CMD_GETF:
			{
				float var = *((float*)(&module->bytecode[pos + 1]));
				printf("%04d: GETF (%f)\n", pos, var);
				pos += 5;
				break;
			}
			case CMD_GETS:
			{
				int var = *((int*)(&module->bytecode[pos + 1]));
				printf("%04d: GETS (%d, \"%s\")\n", pos, var, module->strings[var]);
				pos += 5;
				break;
			}
			case CMD_ADD:
			{
				printf("%04d: ADD\n", pos);
				pos += 1;
				break;
			}
			case CMD_SUB:
			{
				printf("%04d: SUB\n", pos);
				pos += 1;
				break;
			}
			case CMD_MUL:
			{
				printf("%04d: MUL\n", pos);
				pos += 1;
				break;
			}
			case CMD_DIV:
			{
				printf("%04d: DIV\n", pos);
				pos += 1;
				break;
			}
			case CMD_CONCAT:
			{
				printf("%04d: CONCAT\n", pos);
				pos += 1;
				break;
			}
			case CMD_OR:
			{
				printf("%04d: OR\n", pos);
				pos += 1;
				break;
			}
			case CMD_AND:
			{
				printf("%04d: AND\n", pos);
				pos += 1;
				break;
			}
			case CMD_XOR:
			{
				printf("%04d: XOR\n", pos);
				pos += 1;
				break;
			}
			case CMD_CALL:
			{
				int func = *((int*)(&module->bytecode[pos + 1]));
				printf("%04d: CALL (%s, %d arguments, retval: %d)\n",
					pos, module->strings[func], module->bytecode[pos + 5], module->bytecode[pos + 6]);
				pos += 7;
				break;
			}
			case CMD_CALLMEMBER:
			{
				int func = *((int*)(&module->bytecode[pos + 1]));
				printf("%04d: CALLMEMBER (%s, %d arguments, retval: %d)\n",
					pos, module->strings[func], module->bytecode[pos + 5], module->bytecode[pos + 6]);
				pos += 7;
				break;
			}
			case CMD_END:
			{
				printf("%04d: END\n", pos);
				pos += 1;
				break;
			}
			case CMD_RETURN:
			{
				printf("%04d: RETURN (%d)\n", pos, module->bytecode[pos + 1]);
				pos += 2;
				break;
			}
			default:
				printf("%04d: Unknown bytecode!\n", pos);
				pos++;
		}
	}
}

