
/*
Copyright (c) 2008 Mathias Gottschlag

Permission is hereby granted, free of charge, to any person obtaining a copy
of this software and associated documentation files (the "Software"), to deal
in the Software without restriction, including without limitation the rights
to use, copy, modify, merge, publish, distribute, sublicense, and/or sell
copies of the Software, and to permit persons to whom the Software is
furnished to do so, subject to the following conditions:

The above copyright notice and this permission notice shall be included in
all copies or substantial portions of the Software.

THE SOFTWARE IS PROVIDED "AS IS", WITHOUT WARRANTY OF ANY KIND, EXPRESS OR
IMPLIED, INCLUDING BUT NOT LIMITED TO THE WARRANTIES OF MERCHANTABILITY,
FITNESS FOR A PARTICULAR PURPOSE AND NONINFRINGEMENT. IN NO EVENT SHALL THE
AUTHORS OR COPYRIGHT HOLDERS BE LIABLE FOR ANY CLAIM, DAMAGES OR OTHER
LIABILITY, WHETHER IN AN ACTION OF CONTRACT, TORT OR OTHERWISE, ARISING FROM,
OUT OF OR IN CONNECTION WITH THE SOFTWARE OR THE USE OR OTHER DEALINGS IN
THE SOFTWARE.
*/

#include "vm.h"

#include <stdlib.h>
#include <string.h>
#include <stdio.h>

ps_vm_t *create_vm(void)
{
	ps_vm_t *vm = (ps_vm_t*)malloc(sizeof(ps_vm_t));
	memset(vm, 0, sizeof(ps_vm_t));
	return vm;
}
void delete_vm(ps_vm_t *vm)
{
	free(vm->modules);
	free(vm->structs);
	int i;
	for (i = 0; i < vm->funccount; i++)
	{
		free(vm->functions[i].argtypes);
		free(vm->functions[i].name);
	}
	free(vm->functions);
	free(vm);
}

void vm_register_function(ps_vm_t *vm, void *function, const char *name, int argcount, int *argtypes, int rettype)
{
	vm->functions = (ext_func_t*)realloc(vm->functions, sizeof(ext_func_t) * (vm->funccount + 1));
	vm->functions[vm->funccount].function = function;
	vm->functions[vm->funccount].name = strdup(name);
	vm->functions[vm->funccount].argcount = argcount;
	vm->functions[vm->funccount].argtypes = (int*)malloc(argcount * sizeof(int));
	memcpy(vm->functions[vm->funccount].argtypes, argtypes, argcount * sizeof(int));
	vm->functions[vm->funccount].returntype = rettype;
	vm->funccount++;
}
void vm_register_struct(ps_vm_t *vm, struct_info_t *s)
{
	vm->structs = (struct_info_t**)realloc(vm->structs, sizeof(struct_info_t*) * (vm->structcount + 1));
	vm->structs[vm->structcount] = s;
	vm->structcount++;
}

void vm_add_module(ps_vm_t *vm, module_t *module)
{
	vm->modules = (module_t**)realloc(vm->modules, sizeof(module_t*) * (vm->modcount + 1));
	vm->modules[vm->modcount] = module;
	vm->modcount++;
}
typedef void (*FUNCPTR)(void);
void vm_call_function(ps_vm_t *vm, const char *name)
{
	int i;
	int j;
	for (i = 0; i < vm->modcount; i++)
	{
		int nameidx = get_string(vm->modules[i], name);
		for (j = 0; j < vm->modules[i]->funccount; j++)
		{
			if (vm->modules[i]->functions[j].name == nameidx)
			{
				void (*funcptr)(void);
				funcptr = (FUNCPTR)vm->modules[i]->functions[j].call_stub;
				printf("Executing %p.\n", funcptr);
				funcptr();
				printf("Done.\n");
				return;
			}
		}
	}
}
void *vm_get_function_address(ps_vm_t *vm, const char *name)
{
	int i;
	int j;
	// Script function?
	for (i = 0; i < vm->modcount; i++)
	{
		int nameidx = get_string(vm->modules[i], name);
		for (j = 0; j < vm->modules[i]->funccount; j++)
		{
			if (vm->modules[i]->functions[j].name == nameidx)
			{
				return vm->modules[i]->functions[j].call_stub;
			}
		}
	}
	
	// External function
	for (i = 0; i < vm->funccount; i++)
	{
		if (!strcmp(name, vm->functions[i].name))
		{
			return vm->functions[i].function;
		}
	}
	
	return 0;
}
struct_info_t *vm_get_struct_info(ps_vm_t *vm, const char *name)
{
	int i;
	for (i = 0; i < vm->structcount; i++)
	{
		if (!strcmp(name, vm->structs[i]->name))
		{
			return vm->structs[i];
		}
	}
	return 0;
}

